/*
 *  Overlay Javascript for thunderbird
 *
 *  Copyright (c) 2009 Patrik Dufresne & Ruben Verweij
 */

/*
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 */


/**
 * Reference to the interface defined in nsIMessengerNotifications.idl
 */
const nsIMessengerNotifications = Components.interfaces.nsIMessengerNotifications;

/**
 * Reference to the required base interface that all components must support
 */
const nsISupports = Components.interfaces.nsISupports;
const nsIComponentRegistrar = Components.interfaces.nsIComponentRegistrar;
const nsICategoryManager = Components.interfaces.nsICategoryManager;
const nsIStringBundleService = Components.interfaces.nsIStringBundleService;
const nsIFolderListener = Components.interfaces.nsIFolderListener;

/**
 * UUID uniquely identifying our component
 * You can get from: http://kruithof.xs4all.nl/uuid/uuidgen here
 */
const CLASS_ID = Components.ID("{4157dfb0-173f-11de-8c30-0800200c9a66}");

/**
 * Description
 */
const CLASS_NAME = "Messenger Notification Javascript XPCOM Component";

/**
 * Textual unique identifier
 */
const CONTRACT_ID = "@libnofity.info/messenger-notifications;1";

const THUNDERBIRD_ID = "{3550f703-e582-4d05-9a08-453d09bdfdc6}";

const BUNDLE_LOCATION = "chrome://libnotifypopups/locale/libnotifypopups.properties";

const MSG_FLAG_NEW = 0x10000;
const FLR_FLAG_TRASH = 0x0100;
const FLR_FLAG_JUNK = 0x40000000;
const FLR_FLAG_SENTMAIL = 0x0200;
const FLR_FLAG_IMAP_NOSELECT = 0x1000000;
const FLR_FLAG_CHECK_NEW = 0x20000000;
const SRV_RSS = "rss";

/**
 * Class constructor
 */
function MessengerNotifications() {
  this.init();
};

/**
 * Class definition
 */
MessengerNotifications.prototype = {

  init: function() {
    dump("MessengerNotifications::init\r\n");

    // If you only need to access your component from Javascript, uncomment the
    // following line:
    this.wrappedJSObject = this;

    // Retrieve String Bundle
    var sbs = Components.classes["@mozilla.org/intl/stringbundle;1"]
        .getService(nsIStringBundleService);
    this.mBundle = sbs.createBundle(BUNDLE_LOCATION);

    // Initialise array for notification queue
    this.mailQueue = new Array(0);
    // True if a notification is display.
    this.displayingMessage = false;

    // Registering listeners
    var mms = Components.classes[mailSessionContractID]
                            .getService(Components.interfaces.nsIMsgMailSession);
    mms.AddFolderListener(this, nsIFolderListener.added);

  },

  //////////////////////////////////////////////////////////////////////////////
  //// nsIFolderListener
  /**
   * Receive an event when a new mail is add to a folder.
   */
  OnItemAdded: function OnItemAdded(aParentItem, aItem)
  {
    var header = aItem.QueryInterface(Components.interfaces.nsIMsgDBHdr);
    var folder = header.folder;

    if (!this.checkFolder(folder)) {
      return;
    }
    if (header.flags & MSG_FLAG_NEW) {
      var folderName = folder.rootFolder;
      folderName = folderName.prettiestName;
      var subject = header.mime2DecodedSubject
      var author = header.mime2DecodedAuthor;
      this.handleNewMailReceive(folderName, subject, author);
    }
  },

  /**
   * This determines if this folder should even be checked to send a
   * notification to libnotify.
   */
  checkFolder: function checkFolder(aFolder)
  {
    dump("MessengerNotifications::checkFolder\r\n");
    // TODO at least until I come up with some kind of message queue that slowly
    // displays messages, we ignore RSS feeds.

    // We don't check certain folders because they don't contain useful stuff
    if ((aFolder.flags & FLR_FLAG_TRASH) == FLR_FLAG_TRASH ||
        (aFolder.flags & FLR_FLAG_JUNK) == FLR_FLAG_JUNK ||
        (aFolder.flags & FLR_FLAG_SENTMAIL) == FLR_FLAG_SENTMAIL ||
        aFolder.server.type == SRV_RSS)
      return false;

    return true;
  },

  QueryInterface: function QueryInterface(aIID) {
    if (aIID.equals(nsISupports) || aIID.equals(nsIFolderListener)) {
      return this;
    }
    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  /**
   * Put the new mail in a queue ready for notification message.
   */
  handleNewMailReceive : function handleNewMailReceive(folderName, subject, author)
  {
    dump("MessengerNotifications::handleNewMailReceive(" +
        folderName +  ", " + subject + ", " + author + "  )\r\n");

    if(!this.displayingMessage) {
      var item = {'folderName': folderName, 'subject': subject,
          'author': author};
      item.folderName;
      item.subject;
      this.mailQueue[this.mailQueue.length] = item;
      // Wait a bit so that we have more message to display
      setTimeout("libnotifypopups.displayNotification()", 5000);
    }
  },

  /**
   * Build the notification message from mails in queue and display
   * it with notify OSD.
   */
  displayNotification : function displayNotification(){
    this.displayingMessage = true;
    try{
    var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                    .getService(Components.interfaces.nsIPrefService);

    var showFolder = prefs.getBoolPref("libnotifypopups.showFolder");
      if(this.mailQueue.length == 1){

        var subject = this.mailQueue[0].subject;
        var author = this.mailQueue[0].author;
        var folder = this.mailQueue[0].folderName;

        var summary = this.mBundle.GetStringFromName("mail.new.mail.summary");
	if(showFolder){
           var body = this.mBundle.formatStringFromName("mail.new.mails.bodyWithFolder",
            		[subject, author, folder], 3);
	}else{
           var body = this.mBundle.formatStringFromName("mail.new.mails.body",
            		[subject, author], 2);
	}
        this.sendNotification(summary, body, "notification-message-email");

      } else {
	if(this.mailQueue.length != 0){
	  if(this.mailQueue.length == 2){

        var body = "";
        for(var index=0;index<2;index++) { // Due to the if statement we always have 2 elements
          var subject = this.mailQueue[index].subject;
          var author = this.mailQueue[index].author;
	  var folder = this.mailQueue[index].folder;
          if(showFolder){
             body += this.mBundle.formatStringFromName("mail.new.mails.bodyWithFolder",
            		[subject, author, folder], 3) + "\r\n";
	  }else{
             body += this.mBundle.formatStringFromName("mail.new.mails.body",
            		[subject, author], 2) + "\r\n";
	  }
         }
        }
		if(this.mailQueue.length>=3){
		  var body = "";
          for(var index=0;index<2;index++) { // Only display the first two new mails, then display ...
            var subject = this.mailQueue[index].subject;
            var author = this.mailQueue[index].author;
	    var folder = this.mailQueue[index].folder;
            if(showFolder){
             body += this.mBundle.formatStringFromName("mail.new.mails.bodyWithFolder",
            		[subject, author, folder], 3) + "\r\n";
	    }else{
             body += this.mBundle.formatStringFromName("mail.new.mails.body",
            		[subject, author], 2) + "\r\n";
	    }
		  }
          body += "...";
        }

        var summary = this.mBundle.formatStringFromName("mail.new.mails.summary",
            [this.mailQueue.length], 1);
        this.sendNotification(summary, body, "notification-message-email");
	}
	  }
        this.mailQueue = new Array(0);
        this.displayingMessage = false;
    } catch(e){
      this.displayingMessage = false;
      Components.utils.reportError(e);
      throw e;
    }
  },

  /**
   * Send notification using 'notify-send' command line.
   */
  sendNotification : function sendNotification(summary, body, iconName) {
  try {
    dump("MessengerNotifications::sendNotification\r\n");
    var file = Components.classes["@mozilla.org/file/local;1"].
        createInstance(Components.interfaces.nsILocalFile);
    file.initWithPath("/usr/bin/notify-send");

    var process = Components.classes["@mozilla.org/process/util;1"]
        .createInstance(Components.interfaces.nsIProcess);
    process.init(file);
    var args = [utf8.encode(summary), utf8.encode(body), "-i", iconName];
    process.run(false, args, args.length);
  }catch(e){
	var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                        .getService(Components.interfaces.nsIPromptService);
    prompts.alert(null, "Dependency error", "To see notifications via libnotify, the libnotify-bin package needs to be installed. Please install this package via System->Administration->Synaptic Package Manager.");
    Components.utils.reportError(e);
      throw e;
  }
  }
}

/**
 * Initialise the notification system.
 */
var libnotifypopups = null;

function libnotifypopups_onLoad() {
  libnotifypopups = new MessengerNotifications();
  removeEventListener("load", libnotifypopups_onLoad, true);
}

addEventListener("load", libnotifypopups_onLoad, false);

dump("overlay.js\r\n");
